/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal : ProxyRenewalAgent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ProxyRenewalAgent.cpp,v 1.3 2006/08/23 14:02:10 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "ProxyRenewalAgent.h"
#include "glite/data/transfer/agent/scheduler/SchedulerEntry.h"
#include "glite/data/transfer/agent/AgentState.h"
#include "Heartbeat.h"
#include "CleanSDCache.h"

#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/transfer/agent/dao/TSSDAOContext.h"

#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/agents/cred/cred-utility.h"

#include "glite/data/config/service/ParamValue.hxx"
#include "log4cpp/Category.hh"

#include <string>
#include <sstream>
#include <algorithm>
#include <cctype>
#include <unistd.h>

using namespace glite::data::transfer::proxyrenewal;
using namespace glite::data::transfer::agent;
using namespace glite::config;
using namespace log4cpp;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;


/*
 * Constants
 */
namespace {
const char * const PROXYRENEWAL_AGENT_NAME             = "transfer-proxyrenewal-agent";
                                                                                                                          
// Properties Names
const char * const CONTACT_PROPERTY_NAME               = "Contact";
const char * const ACTIONINTERVAL_PROPERTY_NAME        = "DefaultInterval";
    
const char * const RENEW_TYPE_PROPERTY_NAME            = "Renew_Type";
    
const char * const RENEW_INTERVAL_PROPERTY_NAME        = "Renew_Interval";
const char * const HEARTBEAT_INTERVAL_PROPERTY_NAME    = "Heartbeat_Interval";
const char * const CLEANSDCACHE_INTERVAL_PROPERTY_NAME = "CleanSDCache_Interval";

// Properties Default  Values
const unsigned int DEFAULT_ACTION_INTERVAL       = 60;
const unsigned int DEFAULT_HEARTBEAT_INTERVAL    = 60;
const unsigned int DEFAULT_CLEANSDCACHE_INTERVAL = 300;

const char * const DEFAULT_RENEW_TYPE                  = "glite:Renew";

const char * const PROXYRENEWAL_AGENT_TYPE             = "renewal";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new ProxyRenewalAgent();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    ProxyRenewalAgent * agent = dynamic_cast<ProxyRenewalAgent *>(component);
    if(0 != agent){
        delete agent;
    }
}
                                                                                
} // End extern "C"

/*
 * ProxyRenewalAgent
 *
 * Constructor
 */
ProxyRenewalAgent::ProxyRenewalAgent():ComponentConfiguration(PROXYRENEWAL_AGENT_NAME), 
    m_logger(Category::getInstance(PROXYRENEWAL_AGENT_NAME)),
    m_renewType(DEFAULT_RENEW_TYPE),
    m_defaultInterval(DEFAULT_ACTION_INTERVAL),
    m_renewInterval(DEFAULT_ACTION_INTERVAL),
    m_heartbeatInterval(DEFAULT_HEARTBEAT_INTERVAL),
    m_cleansdcacheInterval(DEFAULT_CLEANSDCACHE_INTERVAL){
}
                                                                                                                          
/*
 * ~ProxyRenewalAgent
 *
 * Destructor
 */
ProxyRenewalAgent::~ProxyRenewalAgent(){
}

/*
 * init
 *
 * Initialize the Component
 */
int ProxyRenewalAgent::init(const Params& params){
          
    try{
        Params::const_iterator it;
        // Get Contact
        if((it = params.find(CONTACT_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CONTACT_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_contact = pv->getValue();
        }
        
        // Get Renew Action Type
        if((it = params.find(RENEW_TYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),RENEW_TYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_renewType = pv->getValue();
        }
        // Log Configuration Values
        m_logger.log(Priority::INFO,"%s Initialized. Initialization Parameters are",getName());
        m_logger.log(Priority::INFO,"Contact            : %s", m_contact.c_str());
        m_logger.log(Priority::INFO,"Renew Action Type  : %s", m_renewType.c_str());
    } catch(const LogicError& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Error: %s",getName(),exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Unexpected Exception: %s",getName(),exc.what());
        throw;
    } catch(...) {
        m_logger.log(Priority::ERROR,"%s configuration Unhandled Exception",getName());
        throw;
    }
    return 0;
}

                                                                                                                          
/*
 * config
 *
 * Configure the Component
 */
int ProxyRenewalAgent::config(const Params& params){
    try{
        Params::const_iterator it;
        
        // Get Default Action Interval
        if((it = params.find(ACTIONINTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_defaultInterval = atoi(pv->getValue().c_str());
            if(m_defaultInterval < 1) {
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
            }
        }
        // Get Renew Action Interval
        if((it = params.find(RENEW_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),RENEW_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_renewInterval = atoi(pv->getValue().c_str());
                if(m_renewInterval < 0) {
                    throw ConfigParamException(getName(),RENEW_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_renewInterval = m_defaultInterval;
            }
        } else {
            m_renewInterval = m_defaultInterval;
        }
        // Get Heartbeat Action Interval
        if((it = params.find(HEARTBEAT_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_heartbeatInterval = atoi(pv->getValue().c_str());
                if(m_heartbeatInterval < 0) {
                    throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Get CleanSDCache Action Interval
        if((it = params.find(CLEANSDCACHE_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_cleansdcacheInterval = atoi(pv->getValue().c_str());
                if(m_cleansdcacheInterval < 0) {
                    throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Create the Scheduler
        initScheduler();

        // Log Configuration Values
        m_logger.log(Priority::INFO,"%s Configured. Configuration Parameters are",getName());
        m_logger.log(Priority::INFO,"Default       Action Interval    : %u secs", m_defaultInterval);
        m_logger.log(Priority::INFO,"Renew         Action Interval    : %u secs", m_renewInterval);
        m_logger.log(Priority::INFO,"Heartbeat     Action Interval    : %u secs", m_heartbeatInterval);
        m_logger.log(Priority::INFO,"Clean SDCache Action Interval    : %u secs", m_cleansdcacheInterval);
    } catch(const LogicError& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Error: %s",getName(),exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Unexpected Exception: %s",getName(),exc.what());
        throw;
    } catch(...) {
        m_logger.log(Priority::ERROR,"%s configuration Unhandled Exception",getName());
        throw;
    }    
    try{
        std::string dn = agents::cred::get_proxy_dn();
        m_logger.log(Priority::INFO,"ProxyRenewal Agent uses DN %s",dn.c_str());
    } catch(const std::exception& exc){
        m_logger.log(Priority::WARN,"Cannot Get Agent DN. Reason is %s",exc.what());
    }
    
   m_logger.log(Priority::INFO,"%s Configured",getName());

    return 0;
}

/*
 * start
 *
 * Start the Component
 */
int ProxyRenewalAgent::start(){

    // Start the Scheduler
    if(0 != m_scheduler.get()){
        bool result = m_scheduler->start();
        if(false == result){
            return -1;
        }
    } else {
        return -1;
    }
        
    m_logger.log(Priority::INFO,"ProxyRenewal Agent Started");
    return 0;
}

/*
 * stop
 *
 * Stop the Component
 */
int ProxyRenewalAgent::stop(){
        
    // Stop the Scheduler
    if(0 != m_scheduler.get()){
        m_scheduler->stop();
    } 
    
    m_logger.log(Priority::INFO,"ProxyRenewal Agent Stopped");
    
    return 0;
}


/*
 * fini
 *
 * Finalize the Component
 */
int ProxyRenewalAgent::fini(){
    
    // Finalize the Scheduler
    finiScheduler();
    
    return 0;
}

/*
 * initScheduler
 *
 * Initialize the Scheduler
 */
void ProxyRenewalAgent::initScheduler() /*throw(ConfigException)*/{

    try{
        // Create DAO Context
        if(0 == m_daoCtx.get()){
            m_daoCtx.reset(agents::dao::DAOContextFactory::instance().create());
        }
    } catch(const agents::dao::DAOException& exc){
        m_logger.log(Priority::ERROR,"Can't Create DAO Context. Reason is %s",exc.what());
        throw ConfigurationException(exc.what());
    }
    
    // Create Action Scheduler
    if(0 != m_scheduler.get()){
        finiScheduler();
    }
    m_scheduler.reset(new ActionScheduler(PROXYRENEWAL_AGENT_NAME,*m_daoCtx,this));

    // Register Heartbeat
    scheduleAction(*(Heartbeat::factory()),m_heartbeatInterval);
    
    // Register CleanSDCache
    scheduleAction(*(CleanSDCache::factory()),m_cleansdcacheInterval);
    
	// Renew
	scheduleAction(m_renewType ,m_renewInterval);
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Type
 */
void ProxyRenewalAgent::scheduleAction(const std::string& type, unsigned int interval,const std::string& params) /*throw(ConfigException)*/{
    try{
        // Get Factory Method
        action::ActionFactoryMethod& method = action::ActionFactory::instance().get(type);

        return scheduleAction(method,interval,params);
    } catch(const action::ExecuteException& exc){
        m_logger.log(Priority::ERROR,"Invalid Action type %s. Reason is %s",type.c_str(), exc.what());
        throw ConfigurationException("Unknown Action Type");
    }
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Factory Method
 */
void ProxyRenewalAgent::scheduleAction(action::ActionFactoryMethod& method, unsigned int interval,const std::string& params) /*throw(ConfigException)*/{
    if(interval > 0){
        // Try to create an Action and check that a VO Action
        boost::scoped_ptr<action::Action> a(method.create()); 
        if(0 == dynamic_cast<ProxyRenewalAction*>(a.get())){
            m_logger.log(Priority::ERROR,"Action type %s is not a ProxyRenewal Action",method.type().c_str());
            throw ConfigurationException("Invalid Action Type. Not a ProxyRenewal Action");
        }
        // Add Sceduler Entry
        m_scheduler->add(new SchedulerEntry(method, PROXYRENEWAL_AGENT_NAME, interval, params));
        m_logger.log(Priority::INFO,"Action type %s scheduled",method.type().c_str());
    } else {
        m_logger.log(Priority::INFO,"Action type %s disabled",method.type().c_str());
    }
}

/*
 * finiScheduler
 *
 * Finalize the Scheduler
 */
void ProxyRenewalAgent::finiScheduler(){
    
    // Clear Scheduler Entries
    if(0 != m_scheduler.get()){
        m_scheduler->clear();
    }
    // Delete Scheduler
    m_scheduler.reset();
}

/* 
 * onEventStart
 *
 * Handle Event Start
 */
bool ProxyRenewalAgent::onEventStart() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::cred::DAOFactory> as(*m_daoCtx,
                PROXYRENEWAL_AGENT_NAME,
                PROXYRENEWAL_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            
            // Check if there is an agent already running
            bool agent_active = as.check();
            if(true == agent_active){
                m_logger.log(Priority::ERROR,"Failed to start: An active "\
                    "instance for ProxyRenewal Agent detected");
                return false;
            }
            // Set the Agent Active
            as.set(model::Agent::S_STARTED);
            m_logger.log(Priority::INFO,"Agent State set to Started");
            
            // Register the DAO Context on the TSS 
            dao::g_tss_dao_context.set(m_daoCtx.get());
            m_logger.log(Priority::DEBUG,"DAOContext registered in thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_logger.log(Priority::ERROR,"Failed to Set Agent State to Started: %s",exc.what());
            return false;
        }
    } else {
        m_logger.log(Priority::ERROR,"Invalid Context DAO");
        return false;
    }
    return true;
}

/*
 * onEventStop
 *
 * Handle Event Stop
 */
bool ProxyRenewalAgent::onEventStop() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::cred::DAOFactory> as(*m_daoCtx,
                PROXYRENEWAL_AGENT_NAME,
                PROXYRENEWAL_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            // Set Agent State
            as.update(model::Agent::S_STOPPED);
            m_logger.log(Priority::INFO,"Agent State set to Stopped");
            
            // Deregister the DAO Context from the TSS 
            dao::g_tss_dao_context.set(0);
            m_logger.log(Priority::DEBUG,"DAOContext deregistered from thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_logger.log(Priority::ERROR,"Failed to Set Agent State to Stopped: %s",exc.what());
        }
    } else {
        m_logger.log(Priority::ERROR,"Invalid Context DAO");
    }
    
    return true;
}
