/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal: ProxyRenewalAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ProxyRenewalAction.cpp,v 1.2 2006/08/10 12:04:27 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/proxyrenewal/ProxyRenewalAction.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"
#include "log4cpp/Category.hh"

using namespace glite::data::transfer::proxyrenewal;
using namespace glite::data::transfer::agent;
using namespace glite::data::agents::dao;

using namespace log4cpp;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using glite::data::transfer::agent::action::ExecuteException;

/*
 * ProxyRenewalAction
 *
 * Constructor
 */
ProxyRenewalAction::ProxyRenewalAction(const std::string& name):
        m_logger(Category::getInstance((std::string)"proxyrenewal-action-" + name)),
        m_context(0){
#ifdef _DEBUG
    m_logger.log(Priority::DEBUG,"ProxyRenewalAction %s created",name.c_str());
#endif //_DEBUG            
}

/*
 * ~ProxyRenewalAction
 *
 * Destructor
 */
ProxyRenewalAction::~ProxyRenewalAction(){
#ifdef _DEBUG
    m_logger.log(Priority::DEBUG,"ProxyRenewalAction destroyed");
#endif //_DEBUG            
}

/*
 * context
 *    
 * set DAOContext
 */
void ProxyRenewalAction::context(const std::string& name, DAOContext& ctx){
    m_agentName  = name;
    m_context    = &(ctx);
}

/*
 * context
 *    
 * Get DAOContext
 */
DAOContext& ProxyRenewalAction::context() /*throw (LogicError)*/{
    if(0 == m_context){
        throw LogicError("null context");
    }        
    return *m_context;
}

/*
 * jobDAO
 *    
 * Get a JobDAO Instance
 */
ProxyRenewalAction::JobDAO& ProxyRenewalAction::jobDAO() /*throw (LogicError)*/{
    if(0 == m_jobDAO.get()){
        // Get DAO Instances
        m_jobDAO.reset(dao::cred::DAOFactory::instance().createJobDAO(context()));
    }    
    return *m_jobDAO;
}

/*
 * credDAO
 *    
 * Get a CredDAO Instance
 */
ProxyRenewalAction::CredDAO& ProxyRenewalAction::credDAO() /*throw (LogicError)*/{
    if(0 == m_credDAO.get()){
        // Get DAO Instances
        m_credDAO.reset(dao::cred::DAOFactory::instance().createCredDAO(context()));
    }    
    return *m_credDAO;
}

/*
 * execute
 *
 * Execute the Action
 */
void ProxyRenewalAction::execute(const std::string& params) /*throw (LogicError, ExecuteException)*/{

    m_logger.log(Priority::DEBUG,"Executing Action");
            
    try{
        // Call the template method
        this->exec(params);
    } catch(const DAOException& exc){
        m_logger.log(Priority::ERROR,"DAO Error in Executing Action. Reason is %s", exc.what());
        throw ExecuteException("Some problem found in interacting with DataSource");
    } catch(const ExecuteException& exc){
        m_logger.log(Priority::ERROR,"Failed To Execute Action. Reason is %s", exc.what());
        throw;
    } catch(const RuntimeError& exc){
        m_logger.log(Priority::ERROR,"Failed To Execute Action (RuntimeError). Reason is %s", exc.what());
        throw ExecuteException(exc.what());
    } catch(const LogicError& exc){
        m_logger.log(Priority::ERROR,"LogicError in Executing Action. Reason is %s", exc.what());
        throw;
    } catch(const AgentException& exc){
        m_logger.log(Priority::ERROR,"Unrecognized Exception in Executing Action. Reason is %s", exc.what());
        throw;
    } catch(const std::exception& exc){
        m_logger.log(Priority::ERROR,"Unknown Error in executing action. Reason is %s", exc.what());
        throw;
    } catch(...){
        m_logger.log(Priority::ERROR,"Cannot execute action. Unknown reason");
        throw;
    }
    m_logger.log(Priority::DEBUG,"Action Executed successfully");
}
