/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal: ProxyRenewalConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ProxyRenewalConfig.cpp,v 1.2 2006/08/23 14:02:11 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "ProxyRenewalConfig.h"
#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceException.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/sd-utility.h"
#include "log4cpp/Category.hh"

// Fix Warning
#ifdef IOV_MAX
#undef IOV_MAX
#endif //IOV_MAX
#include "globus_gsi_credential.h"

using namespace glite::data::transfer::proxyrenewal;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;
using glite::data::agents::dao::DAOContext;
using namespace log4cpp;

namespace {    
const char * const PROXYRENEWAL_NAME             = "transfer-proxyrenewal";
    
const char * const MYPROXYSERVER_PROPERTY_NAME   = "MyProxyServer";
const char * const MYPROXYPORT_PROPERTY_NAME     = "MyProxyPort";
const char * const PROXYLIFETIME_PROPERTY_NAME   = "ProxyLifetime";
const char * const MINVALIDITYTIME_PROPERTY_NAME = "MinValidityTime";

const unsigned long DEFAULT_PROXYLIFETIME   = 60 * 60 * 12; // 12 hours
const unsigned long DEFAULT_MINVALIDITYTIME = 60 * 60;      // 1 hour
}

/*
 * instance
 *
 * Retrun ProxyRenewalConfig singleton
 */
ProxyRenewalConfig& ProxyRenewalConfig::instance(){
	static ProxyRenewalConfig s_instance;
	return s_instance;
}


extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return &(ProxyRenewalConfig::instance());
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
}
                                                                                
} // End extern "C"
	
/*
 * ProxyRenewalConfig
 *
 * Constructor
 */
ProxyRenewalConfig::ProxyRenewalConfig():ComponentConfiguration(PROXYRENEWAL_NAME),
    m_logger(Category::getInstance(PROXYRENEWAL_NAME)),
    m_proxyLifetime(DEFAULT_PROXYLIFETIME),
    m_minValidityTime(DEFAULT_MINVALIDITYTIME){
}
                                                                                                                          
/*
 * ~ProxyRenewalConfig
 *
 * Destructor
 */
ProxyRenewalConfig::~ProxyRenewalConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int ProxyRenewalConfig::init(const Params& params){
      
    try{
        Params::const_iterator it;
        
        // Get MyProxy Server Host
        if((it = params.find(MYPROXYSERVER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYSERVER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_myproxyServer = pv->getValue();
        }
                                                                                                                                                          
        // Get My Proxy Server Port
        if((it = params.find(MYPROXYPORT_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYPORT_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_myproxyPort = atol(pv->getValue().c_str());
        }
                                                                                                                                                          
        // Get Proxy Lifetime
        if((it = params.find(PROXYLIFETIME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PROXYLIFETIME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_proxyLifetime = atol(pv->getValue().c_str());
        }
                                                                                                                                                          
        // Get Proxy Lifetime
        if((it = params.find(MINVALIDITYTIME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MINVALIDITYTIME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_minValidityTime = atol(pv->getValue().c_str());
        }

        // Activate Globus GSI Credential Module
        globus_module_activate(GLOBUS_GSI_CREDENTIAL_MODULE);

        // Log Configuration Values
        m_logger.log(Priority::INFO,"%s Initialized. Initialization Parameters are",getName());
        m_logger.log(Priority::INFO,"MyProxyServer   : %s", m_myproxyServer.c_str());
        m_logger.log(Priority::INFO,"MyProxyPort     : %u", m_myproxyPort);
        m_logger.log(Priority::INFO,"ProxyLifetime   : %u", m_proxyLifetime);
        m_logger.log(Priority::INFO,"MinValidityTime : %u", m_minValidityTime);
      
    } catch(const LogicError& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Logic Error: %s",getName(),exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_logger.log(Priority::ERROR,"%s configuration Unexpected Exception: %s",getName(),exc.what());
        throw;
    } catch(...) {
        m_logger.log(Priority::ERROR,"%s configuration Unhandled Exception",getName());
        throw;
    }
    return 0;  
}

                                                                                                                                                          
/*
 * config
 *
 * Configure the Component
 */
int ProxyRenewalConfig::config(const Params& params){
    // Nothing to do
    m_logger.log(Priority::INFO,"%s Configured",getName());
    return 0;
}
                                                                                                                                                          
/*
 * fini
 *
 * Finalize the Component
 */
int ProxyRenewalConfig::fini(){
    // Deativate Globus GSI Credential Module
    globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
                                                                                                                                                          
    m_logger.log(Priority::INFO,"%s Finalized",getName());
    return 0;
}
