/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal : Renew Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Renew.cpp,v 1.6 2006/08/23 14:02:11 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "Renew.h"
#include "ProxyRenewalConfig.h"
#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/agents/dao/dao-types.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/sd-utility.h"
#include "glite/data/agents/sd/sd-predicates.h"
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/UserProxyEnv.h"
#include "glite/data/agents/extra/TempFile.h"
#include "glite/data/agents/extra/Handle.h"

#include "glite/security/proxyrenewal/renewal_core.h"

#include "log4cpp/Category.hh"
#include <boost/scoped_ptr.hpp>
#include <set>

using namespace glite::data::transfer::proxyrenewal;
using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::agents::sd;

using namespace log4cpp;

using glite::data::transfer::agent::model::Cred;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::TempFile;
using glite::data::agents::Handle;

namespace {
const unsigned int MAX_CRED_CHUNCKS = 1000;	
const char * const TMP_DIRECTORY    = "/tmp";
	
/**
 * ProxyRenewalContext class
 */	
class ProxyRenewalContext: boost::noncopyable{
public:

	/**
     * Constructor
     */
    explicit ProxyRenewalContext(glite_renewal_core_context c) throw ():m_ctx(c){
	}
                                                                                                                                                       
    /**
     * Destructor
     */
    ~ProxyRenewalContext() throw () {
        try{
            if(0 != m_ctx){
                glite_renewal_core_destroy_ctx(m_ctx);
            }
        }catch(...){}
    }

    /**
     * Operator*
     */
    glite_renewal_core_context operator*() const {return m_ctx;}
                                                                                                                                                       
    /**
     * Get the Statement Pointer
     */
    glite_renewal_core_context get() const throw() {return m_ctx;}

private:
	/**
     * Default Constructor. Not implemented
     */
	ProxyRenewalContext();
	
private:
	/**
     * The Embedded Contex
     */
	glite_renewal_core_context m_ctx;
};

}

/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( Renew );
      
/*
 * Renew
 *
 * Constructor
 */
Renew::Renew():ProxyRenewalAction("Renew"){
}

/*
 * ~Renew
 *
 * Destructor
 */
Renew::~Renew(){
}

/*
 * execute
 *
 * Execute the Action
 */
void Renew::exec(const std::string& params) /*throw (DAOException, ExecuteException, LogicError)*/{

    // Get Cred DAO
    CredDAO& credDao = credDAO();
    
    // Get the DN-delegId to renew
	CredDAO::StringPairArray ids;
    credDao.getExpiringCreds(ids,ProxyRenewalConfig::instance().minValidityTime(),MAX_CRED_CHUNCKS);

	m_logger.log(Priority::DEBUG,"Got %d credential(s) that are going to expire"\
		" in %d seconds",ids.size(),ProxyRenewalConfig::instance().minValidityTime());
	
    // For each Cred-ID
    CredDAO::StringPairArray::iterator it;
	for(it = ids.begin(); it != ids.end(); ++it){
        
		const std::string& dlg_id  = (*it).first;
		const std::string& user_dn = (*it).second;
		
    	try{
			try{
				// Get the Cred instance
				boost::scoped_ptr<Cred> c(credDao.get(dlg_id,user_dn));
		
				// Renew the credential
				try{
					renewCred(*c);
				} catch(const AgentException&){
					// ORA-03113 WORKAROUND
					// Recheck the DAO Context, since it seems the Myproxy 
					// interation causes the Oracle connection to be dropped
					try{                
						context().check();
					} catch(...){}
					throw;
				}				
				// Start Transaction
				context().start();
				
				// Update the Cred instance
				credDao.update(*c);
				
				// Commit Transaction
				context().commit();
				
				m_logger.log(Priority::INFO,"Credential <%s> for user <%s> renewed",
					c->id().c_str(),c->userDn().c_str());
			} catch(const DAOException& exc){
				m_logger.log(Priority::WARN,"Can't update credential <%s> for "\
					"user <%s>. Error is: %s",dlg_id.c_str(),user_dn.c_str(),exc.what());
			} catch(const RuntimeError& exc){
				m_logger.log(Priority::WARN,"Error is renewing credential "\
					"<%s> for user <%s>. Error is: %s",dlg_id.c_str(),user_dn.c_str(),exc.what());
				throw;				
			} catch(const LogicError& exc){
				m_logger.log(Priority::ERROR,"Logic Error is renewing credential "\
					"<%s> for user <%s>. Error is: %s",dlg_id.c_str(),user_dn.c_str(),exc.what());
				throw;
			}
		} catch(...){
			// Rollback Transaction
        	context().rollback();
			// continue with the next one
			m_logger.log(Priority::ERROR,"Failed to renew credential <%s> for "\
				"user <%s>. Continue with next one",dlg_id.c_str(),user_dn.c_str());
		}
	}
}

/*
 * renewCred
 *
 * Renew the Credentials
 */
void Renew::renewCred(Cred& c) /*throw (ExecuteException, LogicError)*/{
	m_logger.log(Priority::DEBUG,"Renewing credential <%s> for user <%s>",
		c.id().c_str(),c.userDn().c_str());

	// Be sure to always unlink the file (RAII)
    Handle c_f_h;
	TempFile current_proxy_file(
        TempFile::generate("renewal",TMP_DIRECTORY,c_f_h.get()));
	// Store the current proxy into a local file
	writeCertificate(c.certificate(),c_f_h());
    c_f_h.close();
    
	// Get the user DN from the certificate: This call is made since I want to 
	// check that the certificate is not corrupted, otherwise the proxyrenewal
	// code seems to create serious problems to Oracle (e.g. ORA-03113)
	// This may not be nice, but it's the only workaround I've found
	std::string user_dn = agents::cred::get_proxy_dn(current_proxy_file.name());
	
	m_logger.log(Priority::DEBUG,"Current Credential stored in %s. User DN is [%s]",
		current_proxy_file.name().c_str(),user_dn.c_str());

	// Find the Myproxy server & port to be used
	std::string myproxy_server;
	unsigned int myproxy_port = 0;
	getMyProxyServer(c, current_proxy_file.name(), myproxy_server, myproxy_port);

	// Initialize Renewal Context
	glite_renewal_core_context ctx = 0;
	int result = glite_renewal_core_init_ctx(&ctx);
	if(0 != result){
		m_logger.log(Priority::ERROR,"Failed to initialize proxyrenewal context");
		throw ExecuteException("Can't initialize proxyrenewal context");
	}
	ctx->log_dst = GLITE_RENEWAL_LOG_NONE;
	// RAII
	ProxyRenewalContext r_ctx(ctx);

	// Get current time;	
	time_t current;
	time(&current);
	
	// Call the org.security.proxyrenewal library to renew and redecorate 
	// the credential
	char * new_proxy_file_ptr = 0;	
	result = glite_renewal_core_renew((*r_ctx),
					(!myproxy_server.empty())?myproxy_server.c_str():0,
					myproxy_port,current_proxy_file.name().c_str(),&new_proxy_file_ptr);
	// Check Result
	if(0 != result){
		const char * err_msg = (*r_ctx)->err_message;
		// Failed to renew Proxy Certificate
		std::stringstream reason;
		reason << "Failed to renew proxy certificate. Reason: " << 
			((0 != err_msg)?err_msg:"(none)");
		m_logger.log(Priority::DEBUG,reason.str().c_str());
		throw ExecuteException(reason.str());
	}
	if(0 == new_proxy_file_ptr){
		m_logger.log(Priority::DEBUG,"Null Proxy Returned");
		throw LogicError("Null proxy returned");
	}
	// RAII
	std::string new_proxy_file = new_proxy_file_ptr;
	free(new_proxy_file_ptr);
	if(true == new_proxy_file.empty()){
		m_logger.log(Priority::DEBUG,"Empty Proxy Returned");
		throw LogicError("Empty proxy returned");
	}
	m_logger.log(Priority::DEBUG,"New Credential stored in %s",new_proxy_file.c_str());

	// Be sure to always unlink the file
	TempFile n_f_guard(new_proxy_file);

	// Read the new proxy from the created local file
	std::string new_proxy;
	readCertificate(new_proxy_file,new_proxy);
	
	// Get the new proxy lifetime
	time_t lifetime = agents::cred::get_proxy_lifetime(new_proxy_file);
	if(lifetime <= 0){
		m_logger.log(Priority::DEBUG,"New Credential has an invalid lifetime");
		throw ExecuteException("Invalid Lifetime for the new certificate");
	}
	
	// Update the Cred instance
	c.certificate 	 = new_proxy;
	c.expirationTime = current + lifetime;

	m_logger.log(Priority::DEBUG,"New Credential retrieved, valid for %d",lifetime);
}

/*
 * writeCertificate
 *
 * Write a certificate into a file
 */
void Renew::writeCertificate(const std::string& cert, int fd) /*throw (ExecuteException, LogicError)*/{

	// write the certificate into the file
	const char * ptr 	    = cert.data();
	unsigned int bytes_left = cert.length();
	do{
		int result = write(fd, ptr, bytes_left);
		if(result < 0){
			m_logger.log(Priority::ERROR,"Cannot write proxy to temporary file."\
				" Error is: %s",strerror(errno));
			throw RuntimeError("Cannot write proxy certificate");
		}
		// Increment ptr and decrement remaining files
		ptr        += result;
		bytes_left -= result;
	} while(bytes_left > 0);
}

/*
 * readCertificate
 *
 * Read a certificate from a file
 */
void Renew::readCertificate(const std::string& cert_file, std::string& new_cert) /*throw (ExecuteException, LogicError)*/{

	// Open Certificate File
	Handle fd(open(cert_file.c_str(),O_RDONLY));
    if (fd.isNull()) {
        m_logger.log(Priority::ERROR,"Cannot open file <%s>. "\
			"Error is: %s",cert_file.c_str(),strerror(errno));
        throw RuntimeError("Cannot open proxy certificate file");
    }
	
	// Read the Content of the file
	std::stringstream ss;
	char buffer[1024];
	int result = 0;
	do{
		result = read(fd(),buffer,1024);
		if(result < 0){
			m_logger.log(Priority::ERROR,"Cannot read proxy file <%s>. "\
				"Error is: %s",cert_file.c_str(),strerror(errno));
		    throw RuntimeError("Cannot read proxy certificate");
		} else if(result > 0){
			ss.write(buffer,result);
		}
	} while(result > 0);
	
	// Copy the Certificate
	new_cert = ss.str();
}

/*
 * getMyProxyServer
 *
 * Return the myproxy server to be used for the given credentials
 */
void Renew::getMyProxyServer(const Cred& c, const std::string& proxy_file, std::string& myproxy_server, unsigned int& myproxy_port) /*throw ()*/{

	// Check if a MyProxy Server has been configured
	if(false == ProxyRenewalConfig::instance().myproxyServer().empty()){
		myproxy_server = ProxyRenewalConfig::instance().myproxyServer();
		myproxy_port   = ProxyRenewalConfig::instance().myproxyPort();
		m_logger.log(Priority::DEBUG,"Using configured MyProxy server <%s:%d>",
			myproxy_server.c_str(),myproxy_port);
		return;
	}
	
	// Try to see if the user provided a myproxy server in his transfer requests
	std::string submit_host, myproxy_srv, vo_name;
	try{
	    JobDAO& jobDao = jobDAO();
		// Get the last job submitted with this credential
		boost::scoped_ptr<model::Job> job(jobDao.getLast(c.id(), c.userDn()));
		// Get the Job's Vo name
		vo_name = job->voName();
		// Get the submit host 
		submit_host = job->submitHost();
		// Get the specified myproxy server
		myproxy_srv = job->myproxyServer();
        
        m_logger.log(Priority::DEBUG,"Last Job submitted by [%s:%s] "\
            "through <%s> specifies <%s> as MyProxy Server",
            c.id().c_str(), c.userDn().c_str(),submit_host.c_str(),
            myproxy_srv.c_str());
	} catch(const AgentException& exc){
		m_logger.log(Priority::WARN,"Got Error when trying to get the last job"\
            " submitted with credentials <%s:%d>. Error is: %s",
			c.id().c_str(), c.userDn().c_str(),exc.what());
	} catch(...){
		m_logger.log(Priority::WARN,"Got unknown exception when trying to get the last job"\
            " submitted with credentials <%s:%d>",
			c.id().c_str(), c.userDn().c_str());
    }
	
	// Set the user proxy env var for interaction with SD
	UserProxyEnv proxy_env(proxy_file);
	SelectMyProxyForRenewal pred(vo_name);
	if(false == myproxy_srv.empty()){
		// Take the last myproxy server specified by the user
		boost::scoped_ptr<Service> srv(get_service(
			myproxy_srv,SDConfig::instance().myproxy(),vo_name,pred));
		if(0 != srv.get()){
			myproxy_server = srv->hostname;
            myproxy_port = srv->port;
            m_logger.log(Priority::DEBUG,"User specified MyProxy server <%s:%d>",
                myproxy_server.c_str(),myproxy_port);
            return;
        } else {
            m_logger.log(Priority::DEBUG,"No MyProxy server found for <%s>",
                myproxy_srv.c_str());
        }
	} 
	
	// No user specified myproxy Server	
	if(false == submit_host.empty()){
		// Get the myproxy server associated to the FTS which role is 
		// either unset or renewal
		boost::scoped_ptr<Service> srv(get_associated_service(submit_host,
                            SDConfig::instance().fileTransfer(),
                            SDConfig::instance().myproxy(),
                            vo_name,pred));
        if(0 != srv.get()){
			// Get Associated Myproxy Server 
            myproxy_server = srv->hostname;
            myproxy_port = srv->port;
            m_logger.log(Priority::DEBUG,"Found Associated MyProxy server <%s:%d>",
                myproxy_server.c_str(),myproxy_port);
    		return;
        } else {
            m_logger.log(Priority::DEBUG,"No Associated MyProxy server found" \
                " for <%s>",submit_host.c_str());
        }
	}
	
	// Take the first myproxy server returned from SD
    boost::scoped_ptr<Service> srv(get_service_by_type(
        SDConfig::instance().myproxy(),vo_name,pred));
    if(0 != srv.get()){
        // Get Associated Myproxy Server 
        myproxy_server = srv->hostname;
        myproxy_port = srv->port;
        m_logger.log(Priority::DEBUG,"Found MyProxy server <%s:%d>",
            myproxy_server.c_str(),myproxy_port);
        return;
    }
    
    // No Myproxy Server Found
    m_logger.log(Priority::DEBUG,"No MyProxy server found: use the one "\
        "configured in the environemnt");
}
