/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal: ProxyRenewalAgent Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ProxyRenewalAgentTestCase.cpp,v 1.3 2006/08/23 14:02:11 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/dao/cred/CredDAO.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;

using namespace glite::data::agents::dao;

using boost::scoped_ptr;

extern char * g_user_dn;

/**
 * CPPUnit TestCase
 */
class ProxyRenewalAgentTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ProxyRenewalAgentTestCase);
    CPPUNIT_TEST( test );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void test();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ProxyRenewalAgentTestCase, "ProxyRenewalAgentTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning the same credential as
 * the proxyrenewal unit-test
 */
namespace {
const char * const TC_CRED_ID_1			= "my_cred_01";
const char * const TC_CRED_ID_2			= "my_cred_02";
const char * const TC_CRED_ID_3			= "my_cred_03";
const char * const TC_CRED_ID_4			= "my_cred_04";
const char * const TC_CRED_ID_5			= "invalid_cred_05";
const char * const TC_CRED_ID_6			= "my_cred_06";
const char * const TC_CRED_ID_7			= "my_cred_07";
const char * const TC_CRED_ID_8			= "my_cred_08";
const char * const TC_CRED_ID_9			= "my_cred_09";
}

/*
 * setUp
 *
 * set-up test case
 */
void ProxyRenewalAgentTestCase::setUp(){
    
    try{
        // Wait that for a while: agent should finish all the actions
        sleep(30);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test Method
 */
void ProxyRenewalAgentTestCase::test(){
    
    try{
        // Create CredDAO instance
        scoped_ptr<dao::cred::CredDAO> credDAO(dao::cred::DAOFactory::instance().createCredDAO(DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null CredDAO", credDAO.get() != 0);
        
		// Get current time 
		time_t current;
		time(&current);

        // Check the Credentials using CredDAO
		
		// CRED_ID_1
		scoped_ptr<model::Cred> c(credDAO->get(TC_CRED_ID_1,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (1)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (1)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (1)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (1)", c->expirationTime() > (current + 3600));
        
		// CRED_ID_2
		c.reset(credDAO->get(TC_CRED_ID_2,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (2)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (2)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (2)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (2)", c->expirationTime() > (current + 3600));
		
		// CRED_ID_3
		c.reset(credDAO->get(TC_CRED_ID_3,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (3)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (3)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (3)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (3)", c->expirationTime() <= (current + 7200));

		// CRED_ID_4
		c.reset(credDAO->get(TC_CRED_ID_4,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (4)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (4)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (4)", c->expirationTime() < current);

		// CRED_ID_5
		c.reset(credDAO->get(TC_CRED_ID_5,"DN= An Invalid User"));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (5)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (5)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (5)", c->expirationTime() < (current + 3600));

		// CRED_ID_6
		c.reset(credDAO->get(TC_CRED_ID_6,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (6)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (6)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (6)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (6)", c->expirationTime() > (current + 3600));
        
		// CRED_ID_7
		c.reset(credDAO->get(TC_CRED_ID_7,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (7)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (7)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (7)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (7)", c->expirationTime() > (current + 3600));
		
		// CRED_ID_8
		c.reset(credDAO->get(TC_CRED_ID_3,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (8)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (8)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (8)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (8)", c->expirationTime() > (current + 3600));

		// CRED_ID_9
		c.reset(credDAO->get(TC_CRED_ID_9,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (9)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (9)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (9)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (9)", c->expirationTime() > (current + 3600));

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
