/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer ProxyRenewal: Renew Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: RenewTestCase.cpp,v 1.4 2006/08/23 14:02:11 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "Renew.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/transfer/agent/dao/cred/CredDAO.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::proxyrenewal;

using boost::scoped_ptr;

extern char * g_user_dn;

/**
 * CPPUnit TestCase
 */
class RenewTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( RenewTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
private:
    /**
     * The Action Instance
     */
    scoped_ptr<Renew> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RenewTestCase, "RenewTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - My Credential to be renewed:      		  => RENEWED (through the MyProxy 
 *        cred_id     = my_cred_01                 retrieved from SD)
 *        no voms attributes
 *
 * - My Credential to be renewed:      		  => RENEWED (through the MyProxy 
 *        cred_id     = my_cred_02                 retrieved from SD as assoc
 *        with voms attributes                     service of the FTS)
 *
 * - My Credential valid > 1 hour:      	  => NO NEED TO BE RENEW
 *        cred_id     = my_cred_03
 *        with voms attributes
 *
 * - My Credential expired:      		  	  => NOT CONSIDERED
 *        cred_id     = my_cred_04
 *        with voms attributes
 *
 * - Invalid Credential to be renewed:        => FAILED TO RENEW
 *        cred_id     = invalid_cred_05
 *
 * - My Credential expired:      		  	  => RENEWED (through the MyProxy 
 *        cred_id     = my_cred_06                  specified in the job)
 *        with voms attributes
 *
 * - My Credential expired:      		  	  => RENEW (through the MyProxy 
 *        cred_id     = my_cred_08                  retrieved from SD: invalid 
 *        cred_id     = my_cred_07                  MyProxy specified in the job)
 *        with voms attributes
 *
 * - My Credential expired:      		  	  => RENEWED (through the MyProxy 
 *        cred_id     = my_cred_08                  retrieved from SD: submit
 *        with voms attributes                      host without valid MyProxy)
 *
 * - My Credential expired:      		  	  => RENEWED (through the MyProxy 
 *        cred_id     = my_cred_09                  retrieved from SD: submit
 *        with voms attributes                      host without MyProxy)
 *
 */
namespace {
const char * const TC_CRED_ID_1			= "my_cred_01";
const char * const TC_CRED_ID_2			= "my_cred_02";
const char * const TC_CRED_ID_3			= "my_cred_03";
const char * const TC_CRED_ID_4			= "my_cred_04";
const char * const TC_CRED_ID_5			= "invalid_cred_05";
const char * const TC_CRED_ID_6			= "my_cred_06";
const char * const TC_CRED_ID_7			= "my_cred_07";
const char * const TC_CRED_ID_8			= "my_cred_08";
const char * const TC_CRED_ID_9			= "my_cred_09";
}

/*
 * setUp
 *
 * set-up test case
 */
void RenewTestCase::setUp(){
    
    try{
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Renew"));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Renew*>(a.get()) != 0);

        // Store Action        
        m_action.reset(dynamic_cast<Renew*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 *
 * Test Execute Method
 */
void RenewTestCase::testExecute(){
    
    try{
        // Set Context
        m_action->context("proxyrenewal",DaoHelper::ctx());

		// Gte current time 
		time_t current;
		time(&current);
		        
        // Execute Actions
        m_action->execute();
        
        // Create CredDAO instance
        scoped_ptr<dao::cred::CredDAO> credDAO(dao::cred::DAOFactory::instance().createCredDAO(DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null CredDAO", credDAO.get() != 0);
        
        // Check the Credentials using CredDAO
		
		// CRED_ID_1
		scoped_ptr<model::Cred> c(credDAO->get(TC_CRED_ID_1,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (1)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (1)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (1)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (1)", c->expirationTime() > (current + 3600));
        
		// CRED_ID_2
		c.reset(credDAO->get(TC_CRED_ID_2,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (2)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (2)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (2)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (2)", c->expirationTime() > (current + 3600));
		
		// CRED_ID_3
		c.reset(credDAO->get(TC_CRED_ID_3,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (3)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (3)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (3)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (3)", c->expirationTime() <= (current + 7200));

		// CRED_ID_4
		c.reset(credDAO->get(TC_CRED_ID_4,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (4)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (4)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (4)", c->expirationTime() < current);

		// CRED_ID_5
		c.reset(credDAO->get(TC_CRED_ID_5,"DN= An Invalid User"));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (5)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (5)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("New     certificate (5)", c->expirationTime() < (current + 3600));

		// CRED_ID_6
		c.reset(credDAO->get(TC_CRED_ID_6,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (6)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (6)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (6)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (6)", c->expirationTime() > (current + 3600));
        
		// CRED_ID_7
		c.reset(credDAO->get(TC_CRED_ID_7,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (7)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (7)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (7)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (7)", c->expirationTime() > (current + 3600));
		
		// CRED_ID_8
		c.reset(credDAO->get(TC_CRED_ID_3,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (8)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (8)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (8)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (8)", c->expirationTime() > (current + 3600));

		// CRED_ID_9
		c.reset(credDAO->get(TC_CRED_ID_9,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (9)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (9)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (9)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (9)", c->expirationTime() > (current + 3600));

		// CRED_ID_6
		c.reset(credDAO->get(TC_CRED_ID_6,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (6)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (6)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (6)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (6)", c->expirationTime() > (current + 3600));
        
		// CRED_ID_7
		c.reset(credDAO->get(TC_CRED_ID_7,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (7)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (7)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (7)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (7)", c->expirationTime() > (current + 3600));
		
		// CRED_ID_8
		c.reset(credDAO->get(TC_CRED_ID_3,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (8)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (8)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (8)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (8)", c->expirationTime() > (current + 3600));

		// CRED_ID_9
		c.reset(credDAO->get(TC_CRED_ID_9,g_user_dn));
		CPPUNIT_ASSERT_MESSAGE("Null Cred (9)", c.get() != 0);
		CPPUNIT_ASSERT_MESSAGE("Empty   certificate (9)", c->certificate().empty() == false);
		CPPUNIT_ASSERT_MESSAGE("Expired certificate (9)", c->expirationTime() > current);
		CPPUNIT_ASSERT_MESSAGE("Invalid certificate (9)", c->expirationTime() > (current + 3600));

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
